with Text_Io;
use Text_Io;
with Ada.Integer_Text_Io;
use Ada.Integer_Text_Io;
with Ada.Numerics.Discrete_Random;
procedure Absens is
   
   type Pawn_Class is tagged
      record
         Is_Empty   : Boolean := True;
         Is_Special : Boolean := False;
         Verax      : Boolean := False;
      end record;
   
   subtype Position_Range is Natural range 0..12;
   
   type Pawn_Set_Array is array (Position_Range) of Pawn_Class;
      
   Invalid_Move, Not_Empty : exception;
   
   type Absens_Game_Class is
      record
         Player   : Boolean := False;
         Plateau  : Pawn_Set_Array;
         Empty    : Pawn_Class;
         Num      : Natural := 0;
      end record;
   
   procedure Move(Game : in out Absens_Game_Class;
                 Source_Position : in Position_Range;
                 Target_Position : in Position_Range) is
      
   begin
      if Target_Position = 0 then
         if not Game.Empty.Is_Empty then            
            raise Not_Empty;
         end if;
         Game.Empty := Game.Plateau(Source_Position);
         Game.Plateau(Source_Position).Is_Empty := True;         
      elsif Game.Plateau(Source_Position).Verax /= Game.Player then
         raise Invalid_Move;
      elsif not (Game.Plateau(Source_Position).Is_Empty) then         
         if Game.Plateau(Source_Position).Is_Special then
            if Game.Plateau(Target_Position).Verax /= Game.Player or
              Game.Plateau(Target_Position).Is_Empty then
              Game.Plateau(Target_Position) := Game.Plateau(Source_Position);
              Game.Plateau(Source_Position).Is_Empty := True;
              Game.Plateau(Source_Position).Is_Special := False;
            end if;
         elsif Game.Plateau(Target_Position).Is_Empty then
            Game.Plateau(Target_Position) := Game.Plateau(Source_Position);
            Game.Plateau(Source_Position).Is_Empty := True;
            Game.Plateau(Source_Position).Is_Special := False;
         else
            raise Invalid_Move;               
         end if;
      end if;
      Game.Player := not Game.Player;     
   end Move;
   
   
   function Check_End (Game : in Absens_Game_Class) return Boolean is
      Verax : Boolean := False;
      Done : Boolean := True;
   begin
      if not Game.Empty.Is_Empty and 
        Game.Empty.Is_Special then
         Verax := Game.Empty.Verax;
      end if;
      if Verax /= Game.Player then
         return not Done;
      end if;
      for I in 1..Position_Range'Last/2 loop
         if Game.Plateau(I).Verax = Verax then
            null;
         else
            Done := False;
            exit;
         end if;
      end loop;
      if not Done then
         Done := True;
         for I in Position_Range'Last/2+1..Position_Range'Last loop
            if Game.Plateau(I).Verax = Verax then
               null;
            else
               Done := False;
               exit;
            end if;
         end loop;
      end if;
      return Done;
   end Check_End;
   procedure Print(Game : in Absens_Game_Class) is
   begin    
      for Position in 1 .. Position_range'Last/2 loop
         if Game.Plateau(Position).Is_empty then
            Put(' ');
         else
            if Game.plateau(Position).Verax then
               Put('1');
            else
               Put('0');
            end if;
         end if;        
      end loop;
      New_Line;                  
      for position in Position_range'Last/2+1 .. Position_Range'Last loop
         if Game.Plateau(Position).Is_empty then
            Put(' ');
         else
            if Game.plateau(Position).Verax then
               Put('1');
            else
               Put('0');
            end if;
         end if;
      end loop;
      New_Line;
      if Game.Empty.Is_Empty then
         if Game.Empty.Verax then
            Put('1');
         else
            Put('0');
         end if;
      end if;
      New_Line;
      Put(Game.Num);            
   end Print;
   
   
   function System (Cmd : in String) return Integer;
   pragma Import(C, System, "system");
   
   
   Game : Absens_Game_Class;            
   Clear : Integer;
begin
   
   Clear := System("clear" & Character'Val(0));
   
   for Position in 1 .. Position_range'Last/2 loop
      Game.plateau(Position).Verax := True;
      Game.Plateau(Position).Is_empty := False;
   end loop;
   for position in Position_range'Last/2+1 .. Position_Range'Last loop
      Game.Plateau(Position).Verax := False;
      Game.Plateau(Position).Is_empty := False;
   end loop;
   loop
      
      declare
         Position : Natural := 0;
      begin
         
         Put_Line("Choisissez Votre Place :");
         Put("entrez un chiffre de 1 et 8 :");
         Get(Position);
         if not ((Position >= Position_Range'First) and (Position <= Position_Range'Last))  then
            raise Constraint_Error;
         end if;
         Game.Plateau(Position_Range(Position)).Is_Special := True;
         
         exit;   
         
      exception
         when others => null;
      end;
   end loop;   
   
   --  loop
      
   --     declare
   --        Response : Character := 'o';
   --     begin
         
   --        Put("Souhaitez vous débuter le jeu (Oui/non) :");
   --        Get_immediate(Response);
   --        if (Response = 'o' or Response = 'O')  then
   --           Game.Player := True;
   --        end if;
                 
   --        exit;   
         
   --     exception
   --        when others => null;
   --     end;
   --  end loop;   
   
   Clear := System("clear" & Character'Val(0));
   
   
   
   declare
      package Position_Rand is
      new Ada.Numerics.Discrete_Random(Position_Range);         
      
      Rand_1, Rand_2 : Position_Range;
      
      
      use Position_Rand;
      
      Gen_0 : Generator;      
            
      
   begin
      Reset(Gen_0);
      Move(Game, Rand_1, 0);
      Print(Game);
      loop
            delay 0.02;
            Rand_1 := Random(Gen_0);           
            if Game.Plateau(Rand_1).Verax = Game.Player then
               exit;                  
            end if;
         end loop;
         if Game.Empty.Is_Empty then               
            Rand_2 := 0;
         else
            loop
               delay 0.02;
               Rand_2 := Random(Gen_0);
               if Game.Plateau(Rand_2).Is_Empty then
                  exit;
               end if;
            end loop;
         end if;
         
         

         
                                    
         begin
            
            Move(Game, Rand_1, Rand_2);            
            if Game.Player then
               Game.Num := Game.Num + 1;
            end if;
            
            delay 0.05;            
            Clear := System("clear" & Character'Val(0));
            delay 0.05;                        
            Print(Game);                        
            
         exception
            
            when others => 
               null;
            
         end;
         
         
      loop
         
         loop
            delay 0.02;
            Rand_1 := Random(Gen_0);           
            if Game.Plateau(Rand_1).Verax = Game.Player then
               exit;                  
            end if;
         end loop;
         if Game.Empty.Is_Empty then               
            Rand_2 := 0;
         else
            loop
               delay 0.02;
               Rand_2 := Random(Gen_0);
               if Game.Plateau(Rand_2).Is_Empty then
                  exit;
               end if;
            end loop;
         end if;
         
         

         
                                    
         begin
            
            Move(Game, Rand_1, Rand_2);            
            if Game.Player then
               Game.Num := Game.Num + 1;
            end if;
            
            delay 0.05;            
            Clear := System("clear" & Character'Val(0));
            delay 0.05;                        
            Print(Game);                        
	 exception
	    when others =>
	       null;
	       
         end;
         
         
         exit when Check_End(Game);
         
      end loop;
      
   end;
   
   
   
   
end Absens;
